@@ -28,9 +28,9 @@
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-desktop-item.h>
 #include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-xfer.h>
 #include <gconf/gconf-client.h>
 
@@ -1123,11 +1123,14 @@ drag_data_get_menu_cb (GtkWidget        *widget,
 		       guint             time,
 		       GMenuTreeEntry   *entry)
 {
-	char *uri_list;
+	const char *path;
+	char       *uri;
+	char       *uri_list;
 
-	uri_list = g_strconcat (gmenu_tree_entry_get_desktop_file_path (entry),
-				"\r\n",
-				NULL);
+	path = gmenu_tree_entry_get_desktop_file_path (entry);
+	uri = gnome_vfs_get_uri_from_local_path (path);
+	uri_list = g_strconcat (uri, "\r\n", NULL);
+	g_free (uri);
 
 	gtk_selection_data_set (selection_data,
 				selection_data->target, 8, (guchar *)uri_list,
@@ -1153,6 +1156,42 @@ image_menuitem_size_request (GtkWidget      *menuitem,
 	requisition->height = MAX (requisition->height, icon_height);
 }
 
+static char *
+menu_escape_underscores_and_prepend (const char *text)
+{
+	GString    *escaped_text;
+	const char *src;
+	int         inserted;
+	
+	if (!text)
+		return g_strdup (text);
+
+	escaped_text = g_string_sized_new (strlen (text) + 1);
+	g_string_printf (escaped_text, "_%s", text);
+
+	src = text;
+	inserted = 1;
+
+	while (*src) {
+		gunichar c;
+
+		c = g_utf8_get_char (src);
+
+		if (c == (gunichar)-1) {
+			g_warning ("Invalid input string for underscore escaping");
+			return g_strdup (text);
+		} else if (c == '_') {
+			g_string_insert_c (escaped_text,
+					   src - text + inserted, '_');
+			inserted++;
+		}
+
+		src = g_utf8_next_char (src);
+	}
+
+	return g_string_free (escaped_text, FALSE);
+}
+
 void
 setup_menuitem (GtkWidget   *menuitem,
 		GtkIconSize  icon_size,
@@ -1164,10 +1203,10 @@ setup_menuitem (GtkWidget   *menuitem,
 	GtkWidget *label;
 
 	if (create_invisible_mnemonic) {
-		gchar *_title;
+		char *_title;
 
 		label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
-		_title = g_strconcat ("_", title, NULL);
+		_title = menu_escape_underscores_and_prepend (title);
 		gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _title);
 		g_free (_title);
 
@@ -1496,10 +1535,19 @@ create_menuitem (GtkWidget          *menu,
 				     menu_item_targets, 1,
 				     GDK_ACTION_COPY);
 
-		//FIXME should remove .png at the end of the icon name
-		if (gmenu_tree_entry_get_icon (entry) != NULL)
-			gtk_drag_source_set_icon_name (menuitem,
-						       gmenu_tree_entry_get_icon (entry));
+		if (gmenu_tree_entry_get_icon (entry) != NULL) {
+			const char *icon;
+			char       *icon_no_ext;
+
+			icon = gmenu_tree_entry_get_icon (entry);
+			if (!g_path_is_absolute (icon)) {
+				icon_no_ext = panel_util_icon_remove_extension (icon);
+				gtk_drag_source_set_icon_name (menuitem,
+							       icon_no_ext);
+				g_free (icon_no_ext);
+			}
+		}
+
 		g_signal_connect (G_OBJECT (menuitem), "drag_begin",
 				  G_CALLBACK (drag_begin_menu_cb), NULL);
 		g_signal_connect (menuitem, "drag_data_get",
